/*
 * Copyright (c) 2014-2015, TAKAHASHI Tomohiro (TTRFTECH) edy555@gmail.com
 * All rights reserved.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * The software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef __RECEIVER_H__
#define __RECEIVER_H__

#include <arm_math.h>
#include <lpc43xx_gpio.h>

//#define STEREO	1

#define AUDIO_RATE			48000
#define FIR_DECIMATION_RATIO	5
#define IF_RATE				240000
#define CIC_DECIMATION_RATIO	50
#define ADC_RATE			12000000

#define CAPTUREBUFFER_SIZE	64000
#define CAPTUREBUFFER0		((uint8_t*)0x20000000)
#define CAPTUREBUFFER1		((uint8_t*)0x20008000)
#define CAPTUREBUFFER_SIZEHALF	32000

#define NCO_TABLE_SIZE		4000
#define NCO_SAMPLES			4000
#define NCO_SIN_TABLE		((int16_t*)0x1008C000)
#define NCO_COS_TABLE		((int16_t*)0x1008E000)
#define NCO_AMPL			(SHRT_MAX / 128)

#define I_FIR_STATE			((q15_t*)0x10080000)
#define I_FIR_BUFFER		((q15_t*)0x10080100)
#define Q_FIR_STATE			((q15_t*)0x10081000)
#define Q_FIR_BUFFER		((q15_t*)0x10081100)

/*  32000 / 50 = 640 */
#define FIR_BUFFER_SIZE		640
#define FIR_STATE_SIZE		256

/*  640 / 5 = 128 */
#define DEMOD_GAINBITS		0	/* 0 ~ 10 */

#define AUDIO_BUFFER 		((q15_t*)0x1008A000)
#define AUDIO_BUFFER_SIZE	0x2000
#define AUDIO_TEST_BUFFER 	((q15_t*)0x1008C000)

// dsp.c
extern void DMA_IRQHandler(void);
extern void nco_set_frequency(float32_t freq);
extern void generate_test_tone(int freq);
extern void dsp_init();

extern void update_adc_dc_offset(void);
extern void audio_set_gain(int gain);

#define AUDIO_GAIN_MAX 29
#define AUDIO_GAIN_MIN -7
#define AUDIO_GAIN_REF 7

// ui.c
extern void ui_init();
extern void ui_process();

// clkcfg.h
extern void setup_systemclock();
extern void setup_pll0audio(uint32_t msel, uint32_t nsel, uint32_t psel);
extern void setup_i2s_clock(LPC_I2Sn_Type *I2Sx, uint32_t Freq, uint8_t TRMode);


extern volatile int32_t capture_count;
extern int32_t fir_gain;
extern int32_t fir_rshift;

typedef struct {
	uint16_t write_current;
	uint16_t write_total;
	uint16_t read_total;
	uint16_t read_current;
	uint16_t rebuffer_count;
} audio_state_t;

typedef struct {
	q15_t *dest;
	int16_t *nco_base;
	int32_t dest_idx;
	int32_t s0;
	int32_t s1;
	int32_t s2;
	int32_t d0;
	int32_t d1;
	int32_t d2;
	uint32_t dc_offset;
} cic_state_t;

extern cic_state_t cic_i;
extern cic_state_t cic_q;

typedef struct {
	int32_t index;
	uint32_t average;
} am_demod_state_t;

extern am_demod_state_t am_demod_state;


// pwm.c
typedef enum {
	PWM_6 = 6
} PWMName;

struct pwmout_s {
    PWMName pwm;
    uint8_t mr;
};

typedef struct pwmout_s pwmout_t;

void pwmout_init(pwmout_t* obj, int pwm);
void pwmout_write(pwmout_t* obj, int value);
void pwmout_setupdma(uint16_t *buffer, int count);



#define LED_INIT()	     (LPC_GPIO_PORT->DIR[0] |= (1UL << 8))
#define LED_ON()		 (LPC_GPIO_PORT->SET[0] |= (1UL << 8))
#define LED_OFF()		 (LPC_GPIO_PORT->CLR[0] = (1UL << 8))
#define LED_TOGGLE()	 (LPC_GPIO_PORT->NOT[0] = (1UL << 8))

#define ROTLED_INIT()	     (LPC_GPIO_PORT->DIR[1] |= (1UL << 3)|(1UL << 4))
#define ROTLED_RED()		 do{LPC_GPIO_PORT->SET[1] |= (1UL << 3);LPC_GPIO_PORT->CLR[1] = (1UL << 4);}while(0)
#define ROTLED_GREEN()		 do{LPC_GPIO_PORT->SET[1] |= (1UL << 4);LPC_GPIO_PORT->CLR[1] = (1UL << 3);}while(0)
#define ROTLED_OFF()		 (LPC_GPIO_PORT->CLR[1] = (1UL << 3)|(1UL << 4))

#define TESTPOINT_INIT() \
do {scu_pinmux(0x6, 11, PUP_DISABLE | PDN_DISABLE | SLEWRATE_SLOW | FILTER_ENABLE, FUNC0); \
	LPC_GPIO_PORT->DIR[3] |= (1UL << 7); \
	LPC_GPIO_PORT->SET[3] |= (1UL << 7); } while(0)
#define TESTPOINT_ON() 	(LPC_GPIO_PORT->SET[3] |= (1UL << 7))
#define TESTPOINT_OFF()	(LPC_GPIO_PORT->CLR[3] = (1UL << 7))
#define TESTPOINT_TOGGLE()	(LPC_GPIO_PORT->NOT[3] = (1UL << 7))
#define TESTPOINT_SPIKE()	TESTPOINT_TOGGLE();TESTPOINT_TOGGLE()

#define DMA_HALT()	(LPC_GPDMA->C0CONFIG |= (1 << 18))
#define DMA_RUN()	(LPC_GPDMA->C0CONFIG &= ~(1 << 18))

#endif /* __RECEIVER_H__ */
